//
//  lariat-online-lariatfragments/LArIATFragments/WUTFragment.cc
//

#include "WUTFragment.h"
#include "LariatFragment.h"
#include <stdexcept>
#include <sstream>

uint32_t WUTFragment::streamToBuffer(std::ostream& buffer)
{
  uint32_t nchar_written=0;
    
  header.fragmentType = LariatFragment::FRAGMENT_TYPE_WUT_HIT_HEADER;

  using std::streamsize;
  const streamsize headersize=sizeof(WutFrameHeader_t);
  const streamsize hitsize = sizeof(WutHit_t);
  
  header.fragmentSize = headersize + hitsize*header.nHits;

  // write the header

  buffer.write(reinterpret_cast<const char*>(&header),headersize);
  nchar_written+=headersize;
  
  // write the channel data
  if(header.nHits!=hits.size()) {
    throw std::length_error("WUTFragment::streamToBuffer : header.nHits!=hits.size()");
  }
  for(size_t i=0; i<header.nHits; ++i){
    WutHit_t data=hits[i];
    buffer.write(reinterpret_cast<const char*>(&data),hitsize);
    nchar_written+=hitsize;
  }
  return nchar_written;

}


uint32_t WUTFragment::fillFromBuffer(std::istream& buffer){

  uint32_t nchar_read=0;
  
  using std::streamsize;
  const streamsize headersize=sizeof(WutFrameHeader_t);
  const streamsize hitsize = sizeof(WutHit_t);
 
  
  //read the header

  buffer.read(reinterpret_cast<char*>(&header),headersize);
  nchar_read+=headersize;

  //read the channel data

  for(size_t i=0; i<header.nHits; ++i)
  {
    WutHit_t data;
    buffer.read(reinterpret_cast<char*>(&data),hitsize);
    nchar_read+=hitsize;
    hits.push_back(data);
  }
  if(header.nHits!=hits.size()) {
    throw std::length_error("WUTFragment::fillFromBuffer : header.nhits!=hits.size()");
  }  

  return nchar_read;
}

void WUTFragment::print(std::ostream& os)
{
  os << *this;
}

void WUTFragment::print(struct WutFrameHeader *header, std::ostream& os)
{
  os << *header;
}

void WUTFragment::printHit(int h, struct WutHit *hit, std::ostream& os)
{
  char buffer[256];
  snprintf(buffer,256,"  Hit %4d  ", h);
  os << buffer << *hit;
}

std::ostream& operator<<(std::ostream& os, const WUTFragment& e)
{
  std::stringstream b;
  b << e.header << std::endl;
  for(size_t ihit=0; ihit<e.header.nHits; ++ihit)
  { 
    char buffer[256];
    snprintf(buffer,256,"  Hit %4d  ", (int)ihit);
    b << buffer;
    b << e.hits[ihit] << std::endl;
  }
  os << b.str();
  return(os);
}

std::ostream& operator<<(std::ostream& os, const WUTFragment::WutFrameHeader& e)
{
  char buffer[256];
  
  snprintf(buffer,256,"  TimeHeader         0x%8.8X (%d)", e.timeHeader, 
	   static_cast<int>(e.timeHeader) );

  os << "WutHitFrame" << std::endl
     << "  FragmentSize         " << e.fragmentSize << std::endl
     << "  FragmentType         " << e.fragmentType << std::endl
     << "  SpillCounter         " << e.spillCounter << std::endl
     << "  PollCount            " << e.pollCount    << std::endl
     << buffer <<std::endl
     << "  nHits                " << e.nHits;
  return(os);
}

std::ostream& operator<<(std::ostream& os, const WUTFragment::WutHit& e)
{
  char buffer[256];
  snprintf(buffer,256,"Channel 0x%X (%2d)    TimeBin  0x%8.8X (%u)",
	   e.channel, e.channel,
	   e.timeBin, e.timeBin);
  os << buffer;
  return(os);
}

